"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var not_equal_1 = require("bicycle/utils/not-equal");
var useLoadingDuration_1 = require("./useLoadingDuration");
var context_1 = require("./context");
function extractResponse(ctx, result, loaded, errors, errorDetails) {
    if (errors.length !== 0) {
        var r_1 = {
            loaded: false,
            loading: false,
            errored: true,
            result: result,
            errors: errors,
            errorDetails: errorDetails,
            render: function () { return React.createElement(ctx.renderErrors, r_1); },
        };
        return r_1;
    }
    if (!loaded) {
        var r = {
            loaded: false,
            loading: true,
            errored: false,
            result: result,
        };
        return r;
    }
    return {
        loaded: true,
        loading: false,
        errored: false,
        result: result,
    };
}
/**
 * Subscribe to the results of a bicycle query
 *
 * @param query The query to subscribe to
 */
function useQuery(query) {
    var queryRef = React.useRef(query);
    if (not_equal_1.default(queryRef.current, query)) {
        queryRef.current = query;
    }
    var ctx = context_1.useContext();
    var _a = React.useState(function () {
        var r = ctx.client.queryCache(queryRef.current);
        return extractResponse(ctx, r.result, r.loaded, r.errors, r.errorDetails);
    }), response = _a[0], setResponse = _a[1];
    var loadingDuration = useLoadingDuration_1.default(response.loading);
    React.useEffect(function () {
        var ready = false;
        var subscription = ctx.client.subscribe(queryRef.current, function (result, loaded, errors, errorDetails) {
            if (ready) {
                setResponse(extractResponse(ctx, result, loaded, errors, errorDetails));
            }
        });
        ready = true;
        return function () {
            subscription.unsubscribe();
        };
    }, [ctx.client, queryRef.current]);
    if (response.loading) {
        var r_2 = __assign({}, response, { loadingDuration: loadingDuration, render: function () { return React.createElement(ctx.renderLoading, r_2); } });
        return r_2;
    }
    return response;
}
exports.default = useQuery;
//# sourceMappingURL=useQuery.js.map